var port = browser.runtime.connect();

window.addEventListener("message", function(event) {


  // Read extension settings for enabling console log
  browser.storage.local.get(null, function(result) {

    consoleLog = false;

    // Get progress bar init max value
    progressBarInitMaxValue = document.getElementById("signProgress").getAttribute("max");

    // Show baseClass switcher buttons
    document.getElementById("baseClassBtnsLabel").style.display = 'block';
    document.getElementById("baseClassBtnsDiv").style.display = 'block';


    if (result["enableConsoleView"] == 1) {
      consoleLog = true;
    }


    // We only accept messages from ourselves
    if (event.source != window) {
      return;
    }

    if (event.data.method) {

      if (consoleLog)
        console.log("Content script received: " + event.data.method);

      // Custom code for each method call
      if (event.data.method == 'getCardsAndCertificates') {
        controlSignbutton('disable');
        controlCardAndCertificateList('disable');
        clearPKCS11LibPathInfo();
        updateProgressBar('infinite', 'local');
        writeInfo(browser.i18n.getMessage("card_init_in_progress"), 'progress');
      }

      if (event.data.method == 'signData') {
        controlSignbutton('disable');
        controlCancelbutton('disable');
        controlCardAndCertificateList('disableNoClear');
        updateProgressBar('infinite', 'local');
        document.getElementById("signProgressNumberContainer").style.display = 'block';
        writeInfo(browser.i18n.getMessage("signing_in_progress"), 'progress');
      }

      if (event.data.type && event.data.type == 'localCall') {
        // Call function from this content script
        this[event.data.method](event.data.params);
      } else {
        setTimeout(function() {

          // Send message to background script which communicate with native app
          port.postMessage({
            consoleLogEnableView: consoleLog,
            params: null
          });
          port.postMessage(event.data);

        }, 1000);


      }

    }

  });

}, false);


// Listener for messages from background script
browser.runtime.onMessage.addListener(function(response, sender, sendResponse) {

  // Get method name from response
  var methodName = Object.keys(response)[0];

  var responseParams = JSON.stringify(response[methodName]);

  console.log("Receiving response from extension " + responseParams);

  // Check for init errors from native application. Error format: ERROR$Base64encodedErrorString
  // Errors are always base64 encoded
  if (responseParams.search("ERROR") != -1) {
    var errMsgExp = responseParams.split("$");

    // Decode error msg and show msg to user
    // (we can't use atob function because of java base64 encoding and JS decoding Incompatibility)
    writeInfo(base64_decode(errMsgExp[1]), 'error');

    updateProgressBar(progressBarInitMaxValue, 'local');

    document.getElementById("signProgressNumberContainer").style.display = '';

  } else {
    // Call function (method name from response)
    this[methodName](response[methodName]);
  }

});

// Get Native Application version
function getAppVersion(params) {
  var nativeAppRecentVersion = parseFloat(document.getElementById("nativeAppRecentVersion").value);
  var nativeAppUserVersion = parseFloat(params);
  var nativeAppVersionStep = document.getElementById("nativeAppVersionStep").value;

  if (consoleLog)
    console.log("Application version (user): " + nativeAppUserVersion);

  // Checking native app version
  if ((nativeAppRecentVersion > nativeAppUserVersion) && nativeAppVersionStep == 'major') {
    writeOldVersionInfo();
  } else {
    if ((nativeAppRecentVersion > nativeAppUserVersion) && nativeAppVersionStep == 'minor') {
      document.getElementById("newAppUpdateButton").style.display = 'block';
    }

    document.getElementById("signer_preloader").style.display = 'none';
    document.getElementById("signer_native").style.display = 'block';
    var baseClass = document.getElementById("base_class").value;
    port.postMessage({
      "method": "getCardsAndCertificates",
      "params": baseClass
    });
  }
}

// Get list of smartcards and certificates
function getCardsAndCertificates(params) {
  if (consoleLog)
    console.log("Set Cards and Certificates");

  cardsAndCertificates = [];
  var cardList = [];

  var keys = Object.keys(params);

  var i = 0;

  keys.forEach(function(key) {

    cardsAndCertificates[key] = params[key];

    // No Non-Repudiation certs - Skip
		if (Object.keys(cardsAndCertificates[key].cardCertificates).length === 0) {
			return;
		}

    cardList[key] = cardsAndCertificates[key].cardDescription[0];

    i++;
  });

  if (i > 0) {
    setCardList(cardList);
  } else {
    writeInfo(browser.i18n.getMessage("error_no_smartcard_with_sign_cert"), 'error');
  }

}

// Sign data
function signData(params) {
  if (consoleLog)
    console.log("Sign data: " + params);

  document.getElementById("signed_data").value = params;

  writeInfo(browser.i18n.getMessage("saving_signed_data_in_progress"), 'progress');

  saveSignedData(params);

}

// Save signed data (Ajax like POST to server)
function saveSignedData(signedData) {
  
  var csrf_token =  document.getElementsByName("asg_csrf_token")[0].value;
  
  if (consoleLog)
    console.log("Saving signed data: " + signedData);

  var msg;
  xhr = new XMLHttpRequest();

  xhr.open('POST',
    encodeURI(document.getElementById("signed_data_put_path").value));
  xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
  xhr.onload = function() {

    updateProgressBar(progressBarInitMaxValue, 'local');

    if (xhr.status !== 200) {

      writeInfo(browser.i18n.getMessage("saving_sign_data_unknown_error"), 'error');
    } else if (xhr.status === 200) {

      var res = xhr.responseText;
      var resExp = res.split("$");

      if (resExp[0] == "ok") {
        writeInfo(base64_decode(resExp[1]), 'info');

        setTimeout(function() {
          location.href = document.getElementById("redirect_after_sign").value;
        }, 2000);
      } else if (resExp[0] == "ok_partial") {
        controlCancelbutton('enable');
        writeInfo(base64_decode(resExp[1]), 'error');
        document.getElementById("show_error_details").style.display = 'block';
      } else {
        if (consoleLog)
          console.log("Error saving data: " + xhr.responseText);

        controlCancelbutton('enable');
        writeInfo(base64_decode(resExp[1]), 'error');
      }
    }
  };

  if(csrf_token != null && csrf_token != '') {
		xhr.send(encodeURI('sigdata=' + signedData + '&asg_csrf_token=' + csrf_token));
	} else {
		xhr.send(encodeURI('sigdata=' + signedData));
	}
}

// Create list of smartcards
function setCardList(cardList) {
  if (consoleLog)
    console.log("Populating card list");

  var select = document.getElementById("cardList");

  select.textContent = "";

  var i = 0;
  var firstCardParam = null;

  for (var index in cardList) {
    var el = document.createElement("option");
    // Card description is always base64 encoded
    el.text = base64_decode(cardList[index]);
    el.value = index;
    select.appendChild(el);

    if (i == 0) {
      firstCardParam = index;
    }

    i++;
  }

  setCertificateList(firstCardParam);
}

// Create list of certificates (from selected smartcard)
function setCertificateList(card) {
  if (consoleLog)
    console.log("Populating certificate list for " + card);

  var select = document.getElementById("certificateList");

  select.textContent = "";

  var certificateList = [];

  var certificateListJSON = cardsAndCertificates[card].cardCertificates;

  var keys = Object.keys(certificateListJSON);

  keys.forEach(function(key) {
    certificateList[key] = certificateListJSON[key];
  });


  for (var index in certificateList) {
    var el = document.createElement("option");
    // Certificate description is always base64 encoded
    el.text = base64_decode(certificateList[index]);
    el.value = index;
    select.appendChild(el);
  }

  // Show PKCS11LibPath information for selected smartcard
  setPKCS11LibPath(card);
}

// Showing PKCS11LibPath information for selected smartcard
function setPKCS11LibPath(card) {
  if (consoleLog)
    console.log("Showing information about card PKCS11 library path" + cardsAndCertificates[card].cardPKCS11LibPath[0]);

  var element = document.getElementById("cardPKCS11LibPath");

  element.textContent = "";

  element.textContent = cardsAndCertificates[card].cardPKCS11LibPath[0];

  // Check if certificate list is empty
  if (isEmptyCertificateList()) {
    controlSignbutton('disable');
    controlCardList('enable');
    controlCertificateList('disable');
    writeInfo(browser.i18n.getMessage("error_no_nonrep_certificate"), 'error');
  }
  // End of init process - show sign button, card and certificate list.
  else {
    controlSignbutton('enable');
    controlCardAndCertificateList('enable');
    writeInfo(browser.i18n.getMessage("click_to_sign"), 'info');
  }

  document.getElementById("cardListRefreshImg").src = "/img/common/signer/refresh.png";

  updateProgressBar(progressBarInitMaxValue, 'local');
}

// User set new PKCS11 library for smartcard
function loadChooseLibPathFrame(params) {
  if (consoleLog)
    console.log("New PKCS11 library from user " + JSON.stringify(params));

  if (params != "") {
    document.getElementById("cardPKCS11LibPath").textContent = params;
  }

}

// Show info/errors to users
function writeInfo(message, type) {

  if (type == 'error') {
    document.getElementById("signInfo").removeAttribute("class", "infoText");
    document.getElementById("signInfo").setAttribute("class", "errorText");

    updateProgressBar(progressBarInitMaxValue, 'local');
    document.getElementById("signProgressNumberContainer").style.display = '';
    controlCancelbutton('enable');
  } else if (type == 'info') {
    document.getElementById("signInfo").removeAttribute("class", "errorText");
    document.getElementById("signInfo").setAttribute("class", "infoText");
  } else {
    document.getElementById("signInfo").removeAttribute("class", "infoText");
    document.getElementById("signInfo").removeAttribute("class", "errorText");
    document.getElementById("signInfo").setAttribute("class", "progressText");
  }

  var fragment = createDocumentFragment(message);

  var signElementInfo = document.getElementById("signInfo");


  if (fragment == null) {

    signElementInfo.textContent = message;

  } else {

    // Clear before append
    signElementInfo.firstChild.data = '';

    signElementInfo.appendChild(fragment);
  }

}

// Show information about new app version
function writeOldVersionInfo() {

  var message = browser.i18n.getMessage('old_app_version', ['<a href="' + document.getElementById("nativeAppUpdateUrl").value + '">', '</a>', '<br>', '<a href="#" onClick="window.location.reload()">', '</a>']);
  var fragment = createDocumentFragment(message);
  signerPreloaderMsg = document.getElementById("signer_preloader_msg");
  signerPreloaderMsg.firstChild.data = '';
  signerPreloaderMsg.appendChild(fragment);

}

// Create DOM document fragment from html message
function createDocumentFragment(message) {

  var docFragment = null;

  let doc = new DOMParser().parseFromString(message, 'text/html');

  // if clean message - no HTML
  if (doc.body.firstChild.nextSibling == null) {

    return null;

  } else {

    var docFragment = document.createDocumentFragment();

    // DocumentFragment from HTML
    cRow = doc.body.firstChild;
    var docFragment = document.createDocumentFragment();

    console.log(cRow);

    el = document.createTextNode(cRow.data);
    docFragment.appendChild(el);


    // While there is a next sibling, loop
    while (cRow = cRow.nextSibling) {
      var el = null;

      // Element node
      if (cRow.nodeType == 1) {
        // Element <br>
        if (cRow.tagName == "BR") {
          // Create br element
          el = document.createElement('br');
          docFragment.appendChild(el);

        }

				// Element <br>
				if (cRow.tagName == "SPAN") {
					// Create br element
					el = document.createElement('span');
					// Atrributes
          for (let j = 0; j < cRow.attributes.length; j++) {
            el.setAttribute(cRow.attributes[j].name, cRow.attributes[j].value);
          }
					el.appendChild(document.createTextNode(cRow.textContent));
					docFragment.appendChild(el);

				}

        // <a> link
        if (cRow.tagName == "A") {
          // Create <a> element
          el = document.createElement('a');

          // Atrributes
          for (let j = 0; j < cRow.attributes.length; j++) {
            el.setAttribute(cRow.attributes[j].name, cRow.attributes[j].value);
          }

          el.appendChild(document.createTextNode(cRow.text));

          docFragment.appendChild(el);

        }
      }
      // Text node
      if (cRow.nodeType == 3) {
        // Create text element
        el = document.createTextNode(cRow.data);
        docFragment.appendChild(el);
      }


    }
  }

  return docFragment


}

// Enable/Disable sign button
function controlSignbutton(state) {
  if (state == 'enable') {
    document.getElementById("signButton").disabled = false;
  } else if (state == 'disable') {
    document.getElementById("signButton").disabled = true;
  }

}

//Enable/Disable sign button
function controlCancelbutton(state) {
  if (state == 'enable') {
    document.getElementById("cancelButton").disabled = false;
  } else if (state == 'disable') {
    document.getElementById("cancelButton").disabled = true;
  }

}

// Enable/Disable card and certificate list dropdown
function controlCardAndCertificateList(state) {
  controlCardList(state);
  controlCertificateList(state);
}

// Enable/Disable card list dropdown
function controlCardList(state) {
  if (state == 'enable') {
    document.getElementById("cardList").disabled = false;
  } else if (state == 'disable') {
    document.getElementById("cardList").disabled = true;
    document.getElementById("cardList").textContent = '';
  } else if (state == 'disableNoClear') {
    document.getElementById("cardList").disabled = true;
  }
}

// Enable/Disable certificate list dropdown
function controlCertificateList(state) {
  if (state == 'enable') {
    document.getElementById("certificateList").disabled = false;
  } else if (state == 'disable') {
    document.getElementById("certificateList").disabled = true;
    document.getElementById("certificateList").textContent = '';
  } else if (state == 'disableNoClear') {
    document.getElementById("certificateList").disabled = true;
  }
}

//Clear PKCS11 library path info
function clearPKCS11LibPathInfo() {
  document.getElementById("cardPKCS11LibPath").textContent = "";
}

// Update progress bar...
function updateProgressBar(value, local) {
  if (value == 'infinite') {
    document.getElementById("signProgress").removeAttribute("value", value);
  } else {
    document.getElementById("signProgress").setAttribute("value", value);

    // Only native app call
    if (typeof local === 'undefined') {
      document.getElementById("signProgressNumber").textContent = value;
    }
  }

}

// Check if certificate list is empty
function isEmptyCertificateList() {
  var certList = document.getElementById("certificateList");

  try {

    var index = certList.options[certList.selectedIndex].getAttribute('value');

    if (index > 0) {
      return false;
    } else {
      return true;
    }

  } catch (err) {
    return true;
  }
}

// Base64 decode function with UTF-8 decoding
function base64_decode(input) {
  var keyStr = "ABCDEFGHIJKLMNOP" +
    "QRSTUVWXYZabcdef" +
    "ghijklmnopqrstuv" +
    "wxyz0123456789+/" +
    "=";

  var output = "";
  var chr1, chr2, chr3 = "";
  var enc1, enc2, enc3, enc4 = "";
  var i = 0;

  input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");

  do {
    enc1 = keyStr.indexOf(input.charAt(i++));
    enc2 = keyStr.indexOf(input.charAt(i++));
    enc3 = keyStr.indexOf(input.charAt(i++));
    enc4 = keyStr.indexOf(input.charAt(i++));

    chr1 = (enc1 << 2) | (enc2 >> 4);
    chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
    chr3 = ((enc3 & 3) << 6) | enc4;

    output = output + String.fromCharCode(chr1);

    if (enc3 != 64) {
      output = output + String.fromCharCode(chr2);
    }
    if (enc4 != 64) {
      output = output + String.fromCharCode(chr3);
    }

    chr1 = chr2 = chr3 = "";
    enc1 = enc2 = enc3 = enc4 = "";

  } while (i < input.length);

  return decodeURIComponent(escape(output));
}
